package myhr.facescan.myhr_facescan;

import android.util.Log;

import com.luxand.FSDK;
import com.luxand.FSDK.HImage;

public class LuxandSdk {
    static FSDK.HTracker tracker;
    static final int CAMERA_ID = 0;
    static boolean isTrackingMode = false;

    static LowPassFilter pan = new LowPassFilter();
    static LowPassFilter tilt = new LowPassFilter();

    public static int Initialize(String licenseKey) {
        int libCode = FSDK.ActivateLibrary(licenseKey);

        if (libCode == FSDK.FSDKE_OK) {
            libCode = FSDK.Initialize();
        }

        return libCode;
    }

    public static int SetTrackingMode() {
        int[] err_pos = new int[1];
        int libCode = FSDK.SetTrackerMultipleParameters(tracker,
                "RecognizeFaces=false;" +
                        "DetectFacialFeatures=true;" +
                        "HandleArbitraryRotations=true;" +
                        "DetermineFaceRotationAngle=true;" +
                        "InternalResizeWidth=100;" +
                        "FaceDetectionThreshold=5;" +
                        "DetectGender=true;" +
                        "DetectAge=true;" +
                        "DetectExpression=true;" +
                        "DetectAngles=true;", err_pos);

        if (err_pos[0] != 0) {
            Log.i("Failed to SetTrackerMultipleParameters: ", String.valueOf(err_pos[0]));
        }

        return libCode;
//        return FSDK.SetFaceDetectionParameters(false, false, 100);
    }

    public static int SetDetectionMode() {
        int[] err_pos = new int[1];
        int libCode = FSDK.SetTrackerMultipleParameters(tracker,
                "RecognizeFaces=false;" +
                        "DetectFacialFeatures=true;" +
                        "HandleArbitraryRotations=false;" +
                        "DetermineFaceRotationAngle=false;" +
                        "InternalResizeWidth=500;" +
                        "FaceDetectionThreshold=5;" +
                        "DetectGender=false;" +
                        "DetectAge=false;" +
                        "DetectExpression=false;" +
                        "DetectAngles=false;", err_pos);

        if (err_pos[0] != 0) {
            Log.i("Failed to SetTrackerMultipleParameters: ", String.valueOf(err_pos[0]));
        }

        return libCode;
//        return FSDK.SetFaceDetectionParameters(true, false, 500);
    }

    private static void ValidateTracker() {
        if (isTrackingMode) {
            return;
        }

        tracker = new FSDK.HTracker();

        if (FSDK.CreateTracker(tracker) == FSDK.FSDKE_OK) {
            isTrackingMode = true;
            SetTrackingMode();
        }
    }

    private static void ValidateDetector() {
        if (!isTrackingMode) {
            return;
        }

        if (FreeTracker() == FSDK.FSDKE_OK) {
            SetDetectionMode();
        }
    }

    public static int FreeTracker() {
        int libCode = FSDK.FreeTracker(tracker);

        if (libCode == FSDK.FSDKE_OK) {
            tracker = null;
            isTrackingMode = false;
        }

        return libCode;
    }

    public static int[] TrackFace(byte[] buffer) {
        ValidateTracker();

        int[] result = new int[5];
        HImage image = new HImage();
        int libCode = FSDK.LoadImageFromJpegBuffer(image, buffer, buffer.length);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to load image: ", String.valueOf(libCode));
            return result;
        }

        long[] faceCount = new long[5];
        long[] ids = new long[5];
        libCode = FSDK.FeedFrame(tracker, CAMERA_ID, image, faceCount, ids);
        FSDK.FreeImage(image);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to feed frame: ", String.valueOf(libCode));
            return result;
        }

        FSDK.TFacePosition pos = new FSDK.TFacePosition();
        libCode = FSDK.GetTrackerFacePosition(tracker, CAMERA_ID, ids[0], pos);

        if (libCode != FSDK.FSDKE_OK) {
            //Log.i("Failed to get tracker position: ", String.valueOf(libCode));
            return result;
        }

        result[0] = pos.xc;
        result[1] = pos.yc;
        result[2] = pos.w;

        String sAttr[] = new String[]{""};
        libCode = FSDK.GetTrackerFacialAttribute(tracker, 0, ids[0], "Age", sAttr, 256);

        if (libCode == FSDK.FSDKE_OK) {
            result[3] = (byte) Math.round(Double.parseDouble(sAttr[0].split(";")[0].split("=")[1]));
        } else {
            Log.i("Failed to GetTrackerFacialAttribute Age: ", String.valueOf(libCode));
        }

        libCode = FSDK.GetTrackerFacialAttribute(tracker, 0, ids[0], "Gender", sAttr, 256);

        if (libCode == FSDK.FSDKE_OK) {
            result[4] = Double.parseDouble(sAttr[0].split(";")[0].split("=")[1]) > 0.8 ? 1 : 2;
        } else {
            Log.i("Failed to GetTrackerFacialAttribute Gender: ", String.valueOf(libCode));
        }

        return result;
    }

    public static double[] GetFaceAttribute(byte[] buffer, Boolean isExpression) {
        ValidateTracker();

        double[] result = new double[2];
        HImage image = new HImage();
        int libCode = FSDK.LoadImageFromJpegBuffer(image, buffer, buffer.length);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to load image: ", String.valueOf(libCode));
            return result;
        }

        long[] faceCount = new long[5];
        long[] ids = new long[5];
        libCode = FSDK.FeedFrame(tracker, CAMERA_ID, image, faceCount, ids);
        FSDK.FreeImage(image);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to feed frame: ", String.valueOf(libCode));
            return result;
        }

        String sAttr[] = new String[]{""};
        libCode = FSDK.GetTrackerFacialAttribute(tracker, 0, ids[0], isExpression ? "Expression" : "Angles", sAttr, 256);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to GetTrackerFacialAttribute: ", String.valueOf(libCode));
            return result;
        }

        Log.i("============================== GetTrackerFacialAttribute: ", sAttr[0]);

        String attr[] = sAttr[0].split(";");

        // Expression - 0 is Smile, 1 is EyesOpen
        // Angles - 0 is Pan, 1 is Tilt

        if (isExpression) {
            result[0] = Double.parseDouble(attr[0].split("=")[1]);
            result[1] = Double.parseDouble(attr[1].split("=")[1]);
        } else {
            result[0] = pan.pass(Float.parseFloat(attr[1].split("=")[1]));
            result[1] = tilt.pass(Float.parseFloat(attr[2].split("=")[1]));
        }

        return result;
    }

    public static byte[][] GetTemplate(byte[] buffer) {
        ValidateDetector();

        byte[][] templates = new byte[2][];
        HImage image = new HImage();
        int libCode = FSDK.LoadImageFromJpegBuffer(image, buffer, buffer.length);
        FSDK.FreeImage(image);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Failed to load image: ", String.valueOf(libCode));
            return templates;
        }

        FSDK.FSDK_Features eyes = new FSDK.FSDK_Features();
        libCode = FSDK.DetectEyes(image, eyes);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Cannot detect eye: ", String.valueOf(libCode));
            return templates;
        }

        FSDK.FSDK_FaceTemplate ft = new FSDK.FSDK_FaceTemplate();
        FSDK.GetFaceTemplateUsingEyes(image, eyes, ft);
        templates[0] = ft.template;

        FSDK.FSDK_Features face = new FSDK.FSDK_Features();
        libCode = FSDK.DetectFacialFeatures(image, face);

        if (libCode != FSDK.FSDKE_OK) {
            Log.i("Cannot detect face: ", String.valueOf(libCode));
            return templates;
        }

        ft = new FSDK.FSDK_FaceTemplate();
        FSDK.GetFaceTemplateUsingFeatures(image, face, ft);
        templates[1] = ft.template;

        return templates;
    }
}

class LowPassFilter {

    private float a;
    private Float y = null;

    public LowPassFilter() {
        a = 0.325f;
    }

    public float pass(final float x) {
        y = a * x + (1 - a) * (y == null ? x : y);
        return y;
    }

}