import 'dart:ffi';
import 'dart:typed_data';

import 'package:camera/camera.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';

class LuxandController extends GetxController {
  var platform = const MethodChannel("myhr_facescan");
  var isInitialized = false;

  final String licenseKey;

  LuxandController({required this.licenseKey});

  initialize() async {
    if (isInitialized) {
      return;
    }

    DynamicLibrary.open("libfsdk.so");

    int result = await platform
        .invokeMethod("activateLuxand", {'licenseKey': licenseKey});

    if (result == 0) {
      isInitialized = true;
    } else {
      print('CANNOT INITIALIZE LUXAND: ${result}');
    }
  }

  freeResource() async {
    await platform.invokeMethod("freeResource");
  }

  Future<List<int>?> detectFace(CameraImage image) async {
    List<int> strides = Int32List(image.planes.length * 2);
    int index = 0;
    final bytes = image.planes.map((plane) {
      strides[index] = (plane.bytesPerRow);
      index++;
      strides[index] = (plane.bytesPerPixel)!;
      index++;
      return plane.bytes;
    }).toList();

    var result = await platform.invokeMethod<List<int>>("detectFace", {
      'platforms': bytes,
      'height': image.height,
      'width': image.width,
      'strides': strides
    });

    return result;
  }

  Future<List<String>> getFaceTemplate(CameraImage image) async {
    List<int> strides = Int32List(image.planes.length * 2);
    int index = 0;
    final bytes = image.planes.map((plane) {
      strides[index] = (plane.bytesPerRow);
      index++;
      strides[index] = (plane.bytesPerPixel)!;
      index++;
      return plane.bytes;
    }).toList();

    var data = await platform.invokeMethod<List<Object?>>("getFaceTemplate", {
      'platforms': bytes,
      'height': image.height,
      'width': image.width,
      'strides': strides
    });

    var result = [data![0].toString(), data[1].toString()];

    return result;
  }

  Future<List<double>> getFaceAttribute(
      CameraImage image, bool isExpression) async {
    List<int> strides = Int32List(image.planes.length * 2);
    int index = 0;
    final bytes = image.planes.map((plane) {
      strides[index] = (plane.bytesPerRow);
      index++;
      strides[index] = (plane.bytesPerPixel)!;
      index++;
      return plane.bytes;
    }).toList();

    var data = await platform.invokeMethod<List<double>>("getFaceAttribute", {
      'platforms': bytes,
      'height': image.height,
      'width': image.width,
      'strides': strides,
      'isExpression': isExpression
    });

    return data!;
  }
}
